LIBRARY IEEE;
USE work.ALL;
USE IEEE.std_logic_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
-----------------------------------------------------------------------
--  ENTITY: DEC_UP_CNT - Four digit Decimal Counter. Counts from
--                       0000 to 9999. 
--      IN: CLK - Use falling edge as clock.
--          RST - Asyncronously reset count to 0000.
--          EN - Enables CLK. If EN = '1' on falling edge of CLK, COUNT
--               is incremented by 1 unless COUNT = 9999.  
--     OUT: COUNT - Decimal count.
--                  If COUNT = 0009 incremented COUNT = 0010.
--                  If COUNT = 0099 incremented COUNT = 0100.
--                  If COUNT = 0999 incremented COUNT = 1000.
--                  If COUNT = 9999 incremented COUNT = 9999.
-----------------------------------------------------------------------
ENTITY DEC_UP_CNT IS
    PORT (CLK      : IN  STD_LOGIC;
          RST      : IN  STD_LOGIC;
          EN       : IN  STD_LOGIC;
          COUNT    : OUT std_logic_vector(15 downto 0)
         );
END ENTITY DEC_UP_CNT;

ARCHITECTURE arch of DEC_UP_CNT IS
begin
    process(clk)
        variable CNT: std_logic_vector(15 downto 0);
    begin
        if clk'event and clk = '0' then
            if RST = '1' then      -- here is reset is syncronous
                CNT := X"0000";    -- ie. it occurs on the clock edge.
            elsif EN = '1' and cnt < X"9999" then
                cnt := cnt + 1;
                if cnt(3 downto 0) > X"9" then
                    cnt(3 downto 0) := X"0";
                    cnt(7 downto 4) := cnt(7 downto 4) + 1;
                    if cnt(7 downto 4) > X"9" then
                        cnt(7 downto 4) := X"0";
                        cnt(11 downto 8) := cnt(11 downto 8) + 1;
                        if cnt(11 downto 8) > X"9" then
                            cnt(11 downto 8) := X"0";
                            cnt(15 downto 12) := cnt(15 downto 12) + 1;
                        end if;
                    end if;
                end if;
            end if;
            COUNT <= cnt;
        end if;
    end process;
END arch;

ARCHITECTURE arch1 of DEC_UP_CNT IS
begin
    process(clk, RST)
        variable CNT_U: integer range 0 to 9;
        variable CNT_T: integer range 0 to 9;
        variable CNT_H: integer range 0 to 9;
        variable CNT_Th: integer range 0 to 9;
    begin
        if RST = '1' then
            CNT_U  := 0;
            CNT_T  := 0;
            CNT_H  := 0;
            CNT_Th := 0;
        elsif clk'event and clk = '0' and EN = '1' then
            if CNT_U < 9 then
                CNT_U := CNT_U + 1;
            elsif not (CNT_T = 9 and CNT_H = 9 and CNT_Th = 9) then
                CNT_U := 0;
                if CNT_T < 9 then
                    CNT_T := CNT_T + 1;
                else
                    CNT_T := 0;
                    if CNT_H < 9 then
                        CNT_H := CNT_H + 1;
                    else
                        CNT_H := 0;
                        if CNT_Th < 9 then
                            CNT_Th := CNT_Th + 1;
                        end if;
                    end if;
               end if;
            end if;                             
        end if;
        COUNT(3 downto 0) <= conv_std_logic_vector(CNT_U, 4);
        COUNT(7 downto 4) <= conv_std_logic_vector(CNT_T, 4);
        COUNT(11 downto 8) <= conv_std_logic_vector(CNT_H, 4);
        COUNT(15 downto 12) <= conv_std_logic_vector(CNT_Th, 4);
    end process;
END arch1;

LIBRARY IEEE;
USE work.ALL;
USE IEEE.std_logic_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
-----------------------------------------------------------------------
--  ENTITY: NCNT - Outputs a terminal pulse of one clock period every 
--          Nth clock. For the Nth clock period TP = '1' for 
--          other clock periods TC = '0'. (i.e. TP goes to 1 after the
--          falling enge of the N-1 clock and then goes to 0 after the
--          falling edge of the N th clock.
-- GENERIC: N   - Number of clocks required to produce terminal
--          pulse. Assume N > 1.
--      IN: CLK - Use falling edge as clock.
--     OUT: TP - Terminal Pulse. At the Nth CLK TP = '1' for
--          that clock period otherwise TP = '0'.
--   NEEDS: NONE
-----------------------------------------------------------------------
ENTITY NCNT IS
    GENERIC(N: integer := 5);
    PORT (CLK      : IN  STD_LOGIC;
          TP       : OUT STD_LOGIC := '0'
         );
END ENTITY NCNT;

ARCHITECTURE arch OF NCNT IS
BEGIN
    PROCESS(CLK)
        variable COUNT : INTEGER range 1 to N := 2;
    BEGIN
        IF falling_edge(CLK) THEN
             if COUNT = N then
                 COUNT := 1;
                 TP <= '1';
             else
                 COUNT := COUNT + 1;
                 TP <= '0';
             end if;
        end if;
    END PROCESS;
END arch;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
----------------------------------------------------------
--  ENTITY: SSD - Seven segment display driver for S3 board.
--          Outputs hex input to seven segment 
--          display of Spartan 3 board. 
--      IN: Hex - 16 bit Hex input to be displayed
--          DP  - Specifies decimal points to turn on.
--              "0001" turns on rightmost decimal point. 
--          CLK - 50 MHz system clock.
--          EN  - If EN(i) = '0' The ith digit is blanked off 
--                else it is on.
--     OUT: DISP_SEG - output to drive segments of display.
--          DISP_AN  - output to drive anodes of dispaly.
--   NEEDS: NONE
------------------------------------------------------------

entity SSD is
    Port ( clk : in std_logic;
           Hex : in std_logic_vector(15 downto 0);
           Disp_seg : out std_logic_vector(7 downto 0);
           Disp_AN : out std_logic_vector(3 downto 0);
           DP : in std_logic_vector(3 downto 0) := "0000";
           EN : in std_logic := '1');
end SSD;

architecture Behavioral of SSD is
    signal SEL: integer range 0 to 3; -- Selects digit
    signal TP: std_logic;             -- Pulse each 1 ms
    constant N: integer := 50000;     -- 50 MHz divisor for 1 ms
begin
----------------------------------------------------------
-- Generate timing pulse TP every 1 ms 
-- to multiplex seven segment disp.
----------------------------------------------------------
    PROCESS(CLK)
        variable COUNT : INTEGER range 1 to N;
    BEGIN
        IF falling_edge(CLK) THEN
             if COUNT = N then
                 COUNT := 1;
                 TP <= '1';
             else
                 COUNT := COUNT + 1;
                 TP <= '0';
             end if;
        end if;    
    END PROCESS;
---------------------------------------------------------
-- Increment digit selection.
---------------------------------------------------------
    process(CLK)
    begin
        if CLK'event and CLK = '0' then
            if TP = '1' then
                SEL <= (SEL + 1) mod 4;
            end if;
        end if;
    end process;
----------------------------------------------------------
-- Output Hex input to seven segment display
----------------------------------------------------------
    process(EN, SEL, hex, DP)
        variable Dmux : std_logic_vector(3 downto 0);
        variable DPmux : std_logic;
        variable Anode : std_logic_vector(3 downto 0);
    begin
        case SEL is
            when 0 =>
                Dmux := hex(3 downto 0);
                DPmux := DP(0);
                Anode := "1110";  
            when 1 =>
                Dmux := hex(7 downto 4);
                DPmux := DP(1);
                Anode := "1101";  
            when 2 =>
                Dmux := hex(11 downto 8);
                DPmux := DP(2);
                Anode := "1011";  
            when 3 =>
                Dmux := hex(15 downto 12);
                DPmux := DP(3);
                Anode := "0111";  
            when others =>
                null;
        end case;
        ----------------------------------------------
        -- Enable unblanked positions
        ----------------------------------------------
        if EN = '1' then
            DISP_AN <= std_logic_vector(anode);
        else
            DISP_AN <= "1111";
        end if;
        DISP_SEG(7) <= not DPmux;
        ----------------------------------------------
        -- Convert Hex digit to seven segment value
        ----------------------------------------------
        case Dmux is
            when "0000" =>
                DISP_SEG(6 downto 0) <= "0000001"; -- 0
            when "0001" =>
                DISP_SEG(6 downto 0) <= "1001111"; -- 1
            when "0010" =>
                DISP_SEG(6 downto 0) <= "0010010"; -- 2
            when "0011" =>
                DISP_SEG(6 downto 0) <= "0000110"; -- 3
            when "0100" =>
                DISP_SEG(6 downto 0) <= "1001100"; -- 4
            when "0101" =>
                DISP_SEG(6 downto 0) <= "0100100"; -- 5
            when "0110" =>
                DISP_SEG(6 downto 0) <= "0100000"; -- 6
            when "0111" =>
                DISP_SEG(6 downto 0) <= "0001111"; -- 7
            when "1000" =>
                DISP_SEG(6 downto 0) <= "0000000"; -- 8
            when "1001" =>
                DISP_SEG(6 downto 0) <= "0000100"; -- 9
            when "1010" =>
                DISP_SEG(6 downto 0) <= "0001000"; -- A
            when "1011" =>
                DISP_SEG(6 downto 0) <= "1100000"; -- b
            when "1100" =>
                DISP_SEG(6 downto 0) <= "0110001"; -- C
            when "1101" =>
                DISP_SEG(6 downto 0) <= "1000010"; -- d
            when "1110" =>
                DISP_SEG(6 downto 0) <= "0110000"; -- E
            when "1111" =>
                DISP_SEG(6 downto 0) <= "0111000"; -- F
            when others =>
                null;
        end case;
     end process;
end Behavioral; 
